#!/usr/bin/env python

__revision__ = '$Id: autotest.py,v 1.4 2001/09/14 08:52:26 syt Exp $'
import sys, os, re

xmlfile = 'out.xml'
htmlfile = 'out.html'
maxtime = '60' # delay until we consider narval is blocked (in secondes)

class NoNarval(Exception):
    pass


def usage():
    pgmname = sys.argv[0]
    print pgmname, 'is Narval test program [See http://www.logilab.org/narval]'
    print 'USAGE: %s [options] all     run all tests' % pgmname
    print '       %s [options] 001     run test 001' % pgmname
    print '       %s [options] clean   remove temporary files' % pgmname
    print '       %s -h        print this help message and exit' % pgmname
    print 'options:'
    print '   --home=<directory>    use given directory as Narval home'
    try:
        print '                         default is', get_narval_home()
    except Exception, e:
        print e
    print '   --narval=<directory>  use given directory to looking for Engine.py'
    try:
        print '                         default is', get_narval_path()
    except Exception, e:
        print e
    print '   -h                    print this help message and exit'

def get_tmpdir():
    if sys.platform[:3] == 'win':
        return '.'
    else:
        return '/tmp'

def get_narval_path():
    try:
        import narval
        return narval.__path__[0]
    except:
        raise NoNarval("Error: narval isn't in PYTHONPATH")

def get_narval_home():
    try:
        from narval.config import get_home
        return get_home()
    except:
        raise NoNarval("Error: unable to find NARVAL_HOME")

def remove_tmpfiles():
    re1 = re.compile('^narval-test-')
    re2 = re.compile('^mem-')
    tmp = get_tmpdir()
    files = os.listdir(tmp)
    for f in files:
        if re1.match(f) or re2.match(f):
            os.remove(os.path.join(tmp, f))
    
def run_test(tmp, phome, home, testnum):
    from os import spawnv, P_WAIT
    from os.path import join, isfile
    print '*'*80
    print 'Executing test', testnum
    memfile = join(home, 'data','cases','test-'+testnum+'.xml')
    valfile = join(home, 'data','validate','val-'+testnum+'.py')
    tmpfile = join(tmp, 'mem-'+testnum+'.xml')
    result = join(tmp, 'narval-test-'+testnum+'.xml')
    narval = join(phome, 'Engine.py')
#     from time import sleep
#     spawnv(P_NOWAIT, sys.executable,
#            ('python', narval, '--debug',
#             '--quit-when-done', '--home', home,
#             '--load-memory-file', memfile,
#             '--listen-on', '8787',
#             '--save-mem-on-quit', tmpfile))
#     time, inc = 0, 2
#     while time < maxtime:
#         print 'SLEEP', `time`
#         sleep(inc)
#         if isfile(tmpfile):
#             break
#         time += inc
#     print 'WAIK UP'
#     if time >= maxtime:
#         try:
#             from narval import RpcServer
#             from socket import gethostname
#             print 'Max time exceed (%d s), try to close Narval' % maxtime
#             remote_narval = RpcServer.Server("http://%s:%d" % (gethostname(), 8787))
#             remote_narval.shutdown()
#             # be sure narval is dead
#             sleep(15)
#         except Exception,e:
#             print e
#     else:
#         # be sure narval is dead
#         sleep(5)
        
    if spawnv(P_WAIT, sys.executable,
              ('python', narval, '--debug',
               '--quit-when-done', '--home', home,
               '--load-memory-file', memfile,
               '--quit-after', maxtime,
               '--save-mem-on-quit', tmpfile)):
        print 'Error while spawn python'
        print 'Tried with following parameters:'
        print sys.executable, narval, '--home', home, '--quit-after', maxtime,\
              '--load-memory-file', memfile, '--save-mem-on-quit', tmpfile
        return 0
    # validate results
    matches = eval(open(valfile).read())
    from extensions.Selftest.test_validator import ValidateTest
    return ValidateTest('test-'+testnum, tmpfile, matches,
                        open(result, 'w+'))
        

def run_all_tests(phome, home):
    files = os.listdir(os.path.join(home, 'data', 'cases'))
    regex1 = re.compile('^test-')
    regex2 = re.compile('\d\d\d')
    nums = [regex2.findall(s)[0] for s in files if regex1.match(s)]
    tmp = get_tmpdir()
    fnum, snum = 0, 0
    for testnum in nums:
        if run_test(tmp, phome, home, testnum):
            snum += 1
        else: fnum +=1
    make_report(home, fnum, snum)


def run_one_test(phome, home, testnum):
    if run_test(get_tmpdir(), phome, home, testnum):
        make_report(home, 0 ,1)
    else:
        make_report(home, 1 ,0)

def make_report(home, failed_num, success_num):
    xml_f =  open(xmlfile, 'w+')
    total = failed_num + success_num
    print >>xml_f, "<?xml version='1.0' encoding='UTF-8'?>"
    print >>xml_f, '<set>'
    print >>xml_f, "<nbtest>%d</nbtest>" % total
    print >>xml_f, "<nbstest>%d</nbstest>" % success_num
    print >>xml_f, "<nbftest>%d</nbftest>" % failed_num
    # append all file to out.xml
    re1 = re.compile('^narval-test-')
    tmp = get_tmpdir()
    nbtest = 0
    for f in os.listdir(tmp):
        if re1.match(f):
            print >>xml_f, open(os.path.join(tmp, f)).read()
    print >>xml_f, '</set>'
    xml_f.close()
    xml_f = open(xmlfile, 'r')
    html_f = open(htmlfile, 'w+')
    style_f = open(os.path.join(home, 'transforms', 'Selftest', 'out.xslt'))
    # apply xslt
    from xml.xslt.Processor import Processor
    p = Processor()
    p.appendStylesheetStream(style_f)
    p.runStream(xml_f, outputStream=html_f)
    remove_tmpfiles()
    
        
if __name__ == '__main__':
    # process option
    import getopt
    try:
        (opt, args) = getopt.getopt(sys.argv[1:], 'hn:',
                                    ['help', 'home=', 'narval='])
    except getopt.error, e:
        print e
        usage()
        sys.exit(1)
    NARVAL_HOME = ''
    NARVAL_PATH = ''
    for o in opt:
        if o[0] == '-h' or o[0] == '--help':
            usage(pgm)
            sys.exit(0)
        if o[0] == '--home':
            NARVAL_HOME = o[1]
        if o[0] in ('--narval', '-n'):
            NARVAL_PATH = o[1]
    if not NARVAL_PATH:
        try:
            NARVAL_PATH = get_narval_path()
        except Exception,e:
            print e
            sys.exit(1)
    if not NARVAL_HOME:
        try:
            NARVAL_HOME = get_narval_home()
        except Exception,e:
            print e
            sys.exit(1)
    sys.path.insert(0,NARVAL_HOME)
    # look at args which are not an option (only 1 waited)
    if len(args) != 1:
        usage()
        sys.exit(1)
    arg = args[0]
    re0 = re.compile('^\d\d\d$')
    # run test
    if arg == 'all':
        run_all_tests(NARVAL_PATH, NARVAL_HOME)
    elif re0.match(arg):
        run_one_test(NARVAL_PATH, NARVAL_HOME, arg)
#    elif arg == 'report':
#        make_report(NARVAL_HOME, -1, -1)
    elif arg == 'clean':
        remove_tmpfiles()
        os.remove(xmlfile)
        os.remove(htmlfile)
    else:
        print 'Unknown command:', arg 
        usage()
        sys.exit(1)
