# Copyright (c) 2000 LOGILAB S.A. (Paris, FRANCE).
# http://www.logilab.fr/ -- mailto:contact@logilab.fr
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

'''This module is intended to be used with the test suite in Narval/tools/tests/cases.
Do not expect anything useful from the actions defined here.'''


from narval.lib import *

MOD_XML='''<?xml version="1.0" encoding="ISO-8859-1"?><module name='Selftest'>'''

# No OPeration #################################################################

def NOP_f(args) :
    return args.ownerDocument.createElementNS('','outputs')

MOD_XML = MOD_XML + \
          """<action name='NOP' func='NOP_f'>
          </action>
          """
# output an element ##########################################################

def produce_element_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    out.appendChild(doc.createElementNS('','element'))
    return out

MOD_XML = MOD_XML + \
          """<action name='produce_element' func='produce_element_f'>
          <output><match>element</match></output>
          </action>
          """

# output an error element ##########################################################

def produce_error_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    out.appendChild(doc.createElementNS('','error'))
    return out

MOD_XML = MOD_XML + \
          """<action name='produce_error' func='produce_error_f'>
          <output><match>element</match></output>
          </action>
          """

# output a wrong element ##########################################################

MOD_XML = MOD_XML + \
          """<action name='produce_wrong_element' func='produce_element_f'>
          <output><match>another_element</match></output>
          </action>
          """
# output a optional element (actually outputs nothing) ############################

MOD_XML = MOD_XML + \
          """<action name='produce_optional_element' func='NOP_f'>
          <output optional='yes'><match>element</match></output>
          </action>
          """


# output multiple elements ##########################################################

def produce_multiple_elements_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    out.appendChild(doc.createElementNS('','element'))
    out.appendChild(doc.createElementNS('','element'))
    out.appendChild(doc.createElementNS('','element'))
    return out

MOD_XML = MOD_XML + \
          """<action name='produce_multiple_element' func='produce_multiple_elements_f'>
          <output list='yes'><match>element</match></output>
          </action>
          """

# output wrong count of  elements ############################

MOD_XML = MOD_XML + \
          """<action name='produce_wrong_element_count' func='produce_multiple_elements_f'>
          <output><match>element</match></output>
          </action>
          """

# output empty list elements ############################

MOD_XML = MOD_XML + \
          """<action name='produce_empty_list' func='NOP_f'>
          <output list='yes'><match>element</match></output>
          </action>
          """

# output optional list of elements ############################

MOD_XML = MOD_XML + \
          """<action name='produce_optional_list' func='NOP_f'>
          <output list='yes' optional='yes'><match>element</match></output>
          </action>
          """


# Bad behaviour in action ##########################################################

def naughty_action_f(args):
    return None

MOD_XML = MOD_XML + \
          """<action name='naughty_action' func='naughty_action_f'>
          <output list='yes'><match>element</match></output>
          </action>
          """

# Bad behaviour in action : no stub exist in module##########################################################

MOD_XML = MOD_XML + \
          """<action name='naughty_action_2' func='non_existent_f'>
          <output list='yes'><match>element</match></output>
          </action>
          """

# Bad behaviour in action : outputs multiple elements with one wrong type ################################
def produce_wrong_multiple_elements_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    out.appendChild(doc.createElementNS('','element'))
    out.appendChild(doc.createElementNS('','another-element'))
    out.appendChild(doc.createElementNS('','element'))
    return out

MOD_XML = MOD_XML + \
          """<action name='produce_wrong_multiple_element' func='produce_wrong_multiple_elements_f'>
          <output list='yes'><match>element</match></output>
          </action>
          """

# Bad behaviour in action : exception raised ################################
def raise_exception_f(args):
    raise 'Bouahahahahahaha!'

MOD_XML = MOD_XML + \
          """<action name='raise_exception' func='raise_exception_f'/>
          """


# identity function: copies its input to its ouput  ###########################

def identity_f(args):
    doc = args.ownerDocument
    out = doc.createElementNS('','outputs')
    # if the input element has a flag attribute, things will be fine
    # otherwise, we'll get an error
    for c in args.childNodes[:]:
        out.appendChild(c)
    return out

# action expects more than what is specified in the prototype
# so that we can test the context from which the element is selected
MOD_XML = MOD_XML + \
          """<action name='test_input' func='identity_f'>
          <input><match>element</match></input>
          <output><match>element[@flag]</match></output>
          </action>
          """

# copies the input element to the output
MOD_XML = MOD_XML + \
          """<action name='test_input_nice' func='identity_f'>
          <input><match>element</match></input>
          <output><match>element</match></output>
          </action>
          """

# eats a list of arguments
MOD_XML = MOD_XML + \
          """<action name='test_input_nice' func='NOP_f'>
          <input list='yes'><match>element</match></input>
          </action>
          """
















MOD_XML = MOD_XML+'</module>'
